#!/usr/bin/perl -w

# SOURCE: http://www.zarafa.com/wiki/index.php/Monitoring_Zarafa_Outgoingqueue

use strict;
use DBI;
use DBD::mysql;
use Getopt::Long;
use Pod::Usage;

my $configfile = '/etc/zarafa/monitor-outgoing-queue.cf';
my $show_q     = 1; 
my $countonly  = 0;
my $flags      = 3;
my @deletes;
my $help;
my %config;

if (@ARGV > 0) {
   GetOptions(
      'help|?|h' => \$help,
      'd=s'      => \@deletes,
      'f=i'      => \$flags,
      "c!"       => \$countonly,
   ) or pod2usage(1);
}
if ($help) { pod2usage(VERBOSE => 2) }

# check and read config file
if ( -r $configfile ) {
  %config = do $configfile;
} else {
  die("can not read $configfile");
}

if ( $config{DB_HOST} !~ /^[\w.-]+$/) { print "error: DB_HOST has wrong format\n"; }
if ( $config{DB_NAME} !~ /^\w+$/)     { print "error: DB_NAME has wrong format\n"; }
if ( $config{DB_USER} !~ /^\w+$/)     { print "error: DB_USER has wrong format\n"; }

sub ltrim($)
{
   my $string = shift;
   $string =~ s/^\s+//;
   return $string;
}

my $db=DBI->connect("DBI:mysql:$config{DB_NAME};host=$config{DB_HOST}", $config{DB_USER}, $config{DB_PASS});

if (@deletes > 0) {
  $show_q = 0;
  my $d_succes = 0;
  foreach (@deletes) {
    my $deletequery = "DELETE FROM outgoingqueue WHERE hierarchy_id=$_";
    if ($_=="ALL") { $deletequery = "DELETE FROM outgoingqueue" }
    my $q_delete=$db->prepare($deletequery);
    my $retval=$q_delete->execute; 
    if ($retval > 0) {
      $d_succes=$d_succes+$retval;
    } else {
      print "Failed to delete message id: $_\n";
    }
  } 
  print "\n$d_succes messages deleted.\n";
}

if ($countonly == 1) {
  $show_q = 0;
  my $countquery = "SELECT count(*) from outgoingqueue where flags=$flags";
  my $q_count=$db->prepare($countquery);
  $q_count->execute;
  my @mailcount = $q_count->fetchrow();
  print "-- $mailcount[0] Requests.\n";
}

if ($show_q != 1) {
  exit 0;
}

my $q_mailq=$db->prepare("SELECT * FROM outgoingqueue where flags=$flags");
$q_mailq->execute;

sub get_item {
  my($q, $idx) = @_;
  my $sth = $db->prepare($q);
  $sth->execute;
  my @ar = $sth->fetchrow();
  return $ar[$idx];
}

print "-Queue ID-\t-From-\t----Arrival Time----\t-Flags-\t--Recipient----------\n";

my $qcount = 0;
while (my @result = $q_mailq->fetchrow_array()) {
  $qcount++;
  my $HIERARCHYID = $result[1];
  my $FLAGS = $result[2];

  # For Zarafa 6.40                                                                                  
  #my $CTIME = get_item("SELECT createtime from hierarchy where id=$HIERARCHYID", 0);                
  # For Zarafa 7.0.x                                                                                 
  my $CTIME = get_item("select FROM_UNIXTIME((((val_hi << 32) + val_lo)-116444736000000000)/10000000) from properties where tag=0x3008 and hierarchyid=$HIERARCHYID", 0);
                                 
  my $FROM = get_item("SELECT val_string FROM properties where hierarchyid=$HIERARCHYID and tag=16378", 0); 
  my $RCPT = get_item("SELECT val_string FROM properties where hierarchyid=$HIERARCHYID and tag=3588", 0);
  my $CC = get_item("SELECT val_string FROM properties where hierarchyid=$HIERARCHYID and tag=3587", 0);
  my $BCC = get_item("SELECT val_string FROM properties where hierarchyid=$HIERARCHYID and tag=3586", 0);
  my $RCPTALL="$RCPT; $CC; $BCC";

  my @RCPTAR = split(/;/, $RCPTALL); 
  print "$HIERARCHYID\t\t$FROM\t$CTIME\t$FLAGS\t";

  my $i=0;
  foreach (@RCPTAR) {
    if ($i > 0) { print "\t\t\t\t\t\t\t" }
    print ltrim($_)."\n";
    $i++;
  }

}

print "\n-- $qcount Requests.\n";

=head1 NAME

zarafa-monitor-outgoing-queue - show the outgoing zarafa queue in a way
that postfix show its mailqueue

=head1 DESCRIPTION

B<zarafa-monitor-outgoing-queue> connects to the mysql database and will
show the outgoing zarafa queue in a way that postfix show its mailqueue.

Monitoring the outgoingqueue of Zarafa can be done through the mysql client
interface. Each item in the outgoingqueue has a flag, shown in the "flags"
column of each record. This flag can have several values. Each of these
values has a different meaning:

=over 12

=item C<0:>

Local queue -> Outlook still has to submit the item, so that
the spooler will pick it up.

=item C<1:>

Master queue -> Zarafa spooler has to pick the item up and send
it to the smtp server.

=item C<2:>

Do send mail -> Zarafa spooler must place item in "Sent Items"
folder of the user store.

=item C<3:>

Combination of 1 + 2 -> Zarafa spooler must pick up the item
and send to smtp + place the item in the "Sent Items" folder.

=back

When you want to monitor only the items that are already submitted by
Outlook and are waiting to be send to the smtp server, you will have to
monitor for item with flags=3.

=head2 OPTIONS

=over 12

=item C<-c>

Only shows number of items in the outgoing queue.

=item C<-f flag-id>

Shows all items in outgoing queue with flag <flag-id>, default is 3.

=item C<-d queue-id>

Deletes item from outgoing queue with queue id <queue-id>. When used with
"-d ALL" all items from outgoing queue will be deleted.

=item C<-h | --help>

Prints this manual page.

=back

=head2 CONFIGURATION

Confguration of B<zarafa-monitor-outgoing-queue> takes place in
F</etc/zarafa/monitor-outgoing-queue.cf>. Available options
are:

=over 12

=item C<DB_HOST>

Host name or IP address of host which runs the MySQL daemon to which 
B<zarafa-monitor-outgoing-queue> will connect.

=item C<DB_NAME>

Name of database which holds zarafa data.

=item C<DB_USER>

MySQL user name to connect to C<DB_HOST>.

=item C<DB_PASS>

MySQL passwort to connect to C<DB_HOST>.

=back

=head1 SEE ALSO

http://www.zarafa.com/wiki/index.php/Monitoring_Zarafa_Outgoingqueue

=head1 AUTHOR

None named

=head1 LICENSE

None named
